#import "ZinstallPlugin.h"
#import "ZInstallSDK.h"

typedef NS_ENUM(NSUInteger, OpenInstallSDKPluginMethod) {
    OpenInstallSDKMethodInit,
    OpenInstallSDKMethodGetInstallParams,
    OpenInstallSDKMethodReportRegister,
    OpenInstallSDKMethodReportEffectPoint
};

@interface ZinstallPlugin () <ZInstallDelegate>

@property (strong, nonatomic, readonly) NSDictionary *methodDict;

@property (strong, nonatomic) FlutterMethodChannel * flutterMethodChannel;

@end

static FlutterMethodChannel * FLUTTER_METHOD_CHANNEL;

@implementation ZinstallPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"zinstall"
            binaryMessenger:[registrar messenger]];
  ZinstallPlugin* instance = [[ZinstallPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

// - (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
//   if ([@"getPlatformVersion" isEqualToString:call.method]) {
//     result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
//   } else {
//     result(FlutterMethodNotImplemented);
//   }
// }
- (instancetype)init {
    self = [super init];
    if (self) {
        [self initData];
        [ZInstallSDK registerXInstallSDK:@"123" delegate:self];
    }
    return self;
}

- (void)initData {
    _methodDict = @{
                    @"registerWakeup"         :      @(OpenInstallSDKMethodInit),
                    @"getInstall"             :      @(OpenInstallSDKMethodGetInstallParams),
                    @"reportRegister"         :      @(OpenInstallSDKMethodReportRegister),
                    @"reportEffectPoint"      :      @(OpenInstallSDKMethodReportEffectPoint)
                    };
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"getPlatformVersion" isEqualToString:call.method]) {
        result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
        return;
    }
    else if ([@"getPasteboardString" isEqualToString:call.method]) {
        result([[UIPasteboard generalPasteboard]string]);
        return;
    }
    // else if ([@"registerApp" isEqualToString:call.method]) {
    //     [ZInstallSDK reportRegister];
    //     result(@"abcde");
    // }
    // else {
    //     result(FlutterMethodNotImplemented);
    // }
        NSNumber *methodType = self.methodDict[call.method];
    if (methodType) {
        switch (methodType.intValue) {
            case OpenInstallSDKMethodInit:
            {
                break;
            }
            case OpenInstallSDKMethodGetInstallParams:
            {
                int time = (int) call.arguments[@"timeout"];
                if (time <= 0) {
                    time = 8;
                }
                [ZInstallSDK getInstallParams:^(NSDictionary * _Nullable dic) {
                    [self installParamsResponse:dic];
                }];
                break;
            }
            case OpenInstallSDKMethodReportRegister:
            {
                [ZInstallSDK reportRegister];
                break;
            }
            case OpenInstallSDKMethodReportEffectPoint:
            {
                NSDictionary * args = call.arguments;
                NSNumber * pointValue = (NSNumber *) args[@"pointValue"];
//                [ZInstallSDK reportEffectPoint:(NSString *)args[@"pointId"] effectValue:[pointValue longValue]];
                break;
            }
            default:
            {
                break;
            }
        }
    } else {
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - Openinstall Notify Flutter Mehtod
- (void)installParamsResponse:(NSDictionary *) appData {
//    NSDictionary *args = [self convertInstallArguments:appData];
    [self.flutterMethodChannel invokeMethod:@"onInstallNotification" arguments:appData];
}

- (void)wakeUpParamsResponse:(NSDictionary *) appData {
//    NSDictionary *args = [self convertInstallArguments:appData];
    [self.flutterMethodChannel invokeMethod:@"onWakeupNotification" arguments:appData];
}

//- (NSDictionary *)convertInstallArguments:(OpeninstallData *) appData {
//    NSString *channelCode = @"";
//    NSString *bindData = @"";
//    if (appData.channelCode != nil) {
//        channelCode = appData.channelCode;
//    }
//    if (appData.data != nil) {
//        bindData = [self jsonStringWithObject:appData.data];
//    }
//    NSDictionary * dict = @{@"channelCode":channelCode, @"bindData":bindData};
//    return dict;
//}

- (NSString *)jsonStringWithObject:(id)jsonObject {
    id arguments = (jsonObject == nil ? [NSNull null] : jsonObject);
    NSArray* argumentsWrappedInArr = [NSArray arrayWithObject:arguments];
    NSString* argumentsJSON = [self cp_JSONString:argumentsWrappedInArr];
    argumentsJSON = [argumentsJSON substringWithRange:NSMakeRange(1, [argumentsJSON length] - 2)];
    return argumentsJSON;
}

- (NSString *)cp_JSONString:(NSArray *)array {
    NSError *error = nil;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:array options:0 error:&error];
    NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    if ([jsonString length] > 0 && error == nil){
        return jsonString;
    } else {
        return @"";
    }
}

#pragma mark - Openinstall API
//通过OpenInstall获取已经安装App被唤醒时的参数（如果是通过渠道页面唤醒App时，会返回渠道编号）
-(void)getWakeUpParams:(NSDictionary *) appData{
    [self wakeUpParamsResponse:appData];
}

+ (BOOL)handLinkURL:(NSURL *) url {
    return [ZInstallSDK handLinkURL:url];
}

+ (BOOL)continueUserActivity:(NSUserActivity *) userActivity {
    return [ZInstallSDK continueUserActivity:userActivity];
}

@end

